;; See the file LICENCE for licence information.
(in-package :ucw)

;;;; * UCW Moduler Application API - This enables us to mix several
;;;; features of different application together like i18n & cookie.
;;;; Also, an end-user can define his/her own context mixin class to
;;;; provide a modular context which is crucial for real applications.

(defclass modular-application-mixin ()
  ()
  (:documentation "Superclass for all application mixins."))

(defgeneric effective-context-definition (modular-application-mixin)
  (:documentation "Returns the name of the effective context mixin class name.")
  (:method-combination list))

(defclass modular-application (standard-application)
  ())

(defmethod effective-context-definition list ((app modular-application))
  'standard-request-context)

(defmethod make-request-context ((app modular-application)
				 (request request)
				 (response response))
  (let* ((context-superclasses (effective-context-definition app))
	 (context-class-symbol (intern (format nil "~A-REQUEST-CONTEXT" (class-name-of app))))
	 (context-class (eval `(defclass ,context-class-symbol ,context-superclasses
				 ()))))
    (make-instance context-class
		   :request request
		   :response response
		   :application app)))
